//-----------------------------------------------------------------------------
/// \file	Main.cpp
/// \date	12-Jan-2005
/// \author	Rob Bateman
/// \brief	This example uses a different form of message, the MSceneMessage
/// 		class. These notifications tend to be the ones related most
/// 		to opening / closing files, and rendering files. You may find that
/// 		whenever someone creates a new scene, you will want to unitilise
/// 		some globally allocated data. Using the scene messages, you can
/// 		retrieve notification of this and act accordingly. The Scene
/// 		message type can handle the following messages :
///
///		MSceneMessage::kSceneUpdate
///			called after any operation that changes which files are loaded
///
///		MSceneMessage::kBeforeNew
///			called before a File > New operation
///
///		MSceneMessage::kAfterNew
///			called after a File > New operation
///
///		MSceneMessage::kBeforeImport
///			called before a File > Import operation
///
///		MSceneMessage::kAfterImport
///			called after a File > Import operation
///
///		MSceneMessage::kBeforeOpen
///			called before a File > Open operation
///
///		MSceneMessage::kAfterOpen
///			called after a File > Open operation
///
///		MSceneMessage::kBeforeExport
///			called before a File > Export operation
///
///		MSceneMessage::kAfterExport
///			called after a File > Export operation
///
///		MSceneMessage::kBeforeSave
///			called before a File > Save (or SaveAs) operation
///
///		MSceneMessage::kAfterSave
///			called after a File > Save (or SaveAs) operation
///
///		MSceneMessage::kBeforeReference
///			called before a File > Reference operation
///
///		MSceneMessage::kAfterReference
///			called after a File > Reference operation
///
///		MSceneMessage::kBeforeRemoveReference
///			called before a File > RemoveReference operation
///
///		MSceneMessage::kAfterRemoveReference
///			called after a File > RemoveReference operation
///
///		MSceneMessage::kBeforeImportReference
///			called before a File > ImportReference operation
///
///		MSceneMessage::kAfterImportReference
///			called after a File > ImportReference operation
///
///		MSceneMessage::kBeforeExportReference
///			called before a File > ExportReference operation
///
///		MSceneMessage::kAfterExportReference
///			called after a File > ExportReference operation
///
///		MSceneMessage::kBeforeSoftwareRender
///			called before a Software Render begins
///
///		MSceneMessage::kAfterSoftwareRender
///			called after a Software Render ends
///
///		MSceneMessage::kBeforeSoftwareFrameRender
///			called before each frame of a Software Render
///
///		MSceneMessage::kAfterSoftwareFrameRender
///			called after each frame of a Software Render
///
///		MSceneMessage::kSoftwareRenderInterrupted
///			called when an interactive render is interrupted by the user
///
///		MSceneMessage::kMayaInitialized
///			called on interactive or batch startup after initialization
///
///		MSceneMessage::kMayaExiting
///			called just before Maya exits
///
//-----------------------------------------------------------------------------

// required to link the libraries
#ifdef WIN32
	#define NT_PLUGIN
	#pragma comment(lib,"Foundation.lib")
	#pragma comment(lib,"OpenMaya.lib")
	#pragma comment(lib,"OpenMayaFx.lib")
	#pragma comment(lib,"OpenMayaUi.lib")
	#pragma comment(lib,"Image.lib")
	#pragma comment(lib,"OpenMayaAnim.lib")
#endif

// under WIN32 we have to 'export' these functions so that they are visible
// inside the dll. If maya can't see the functions, it can't load the plugin!
#ifdef WIN32
	#define EXPORT __declspec(dllexport)
#else
	#define EXPORT
#endif

#include <maya/MFnPlugin.h>
#include <maya/MSceneMessage.h>
#include <maya/MGlobal.h>
#include <maya/MString.h>


//---------------------------------------------------------------------------------------------
///	\brief	This class provides a pre-built way of getting callback notifications of when
/// 		specific File Menu functions are called. By simply deriving this class and
/// 		overiding 1 or more virtuals, you can handle different event types. These are
/// 		often flags often needed for shonky games tools!
///
class FileMenuWatcher {
public:

	/// \brief	ctor	-	creates the scene callbacks
	///
	FileMenuWatcher();

	/// dtor	-	unregisters the callback functions
	///
	~FileMenuWatcher();

protected:

	// export notifications, overide in derived classes
	virtual void PreExport() {
		MGlobal::displayInfo("PreExport");
	}
	virtual void PostExport() {
		MGlobal::displayInfo("PostExport");
	}

	// import notifications, overide in derived classes
	virtual void PreImport() {
		MGlobal::displayInfo("PreImport");
	}
	virtual void PostImport(){
		MGlobal::displayInfo("PostImport");
	}

	// new scene notifications, overide in derived classes
	virtual void PreNew(){
		MGlobal::displayInfo("PreNew");
	}
	virtual void PostNew(){
		MGlobal::displayInfo("PostNew");
	}

	// open scene notifications, overide in derived classes
	virtual void PreOpen(){
		MGlobal::displayInfo("PreOpen");
	}
	virtual void PostOpen(){
		MGlobal::displayInfo("PostOpen");
	}

	// save scene notifications, overide in derived classes
	virtual void PreSave(){
		MGlobal::displayInfo("PreSave");
	}
	virtual void PostSave(){
		MGlobal::displayInfo("PostSave");
	}

private:


	// the callback ID's for export
	MCallbackId m_cb_PreExport;
	MCallbackId m_cb_PostExport;

	// the callback ID's for import
	MCallbackId m_cb_PreImport;
	MCallbackId m_cb_PostImport;

	// the callback ID's for new scene
	MCallbackId m_cb_PreNew;
	MCallbackId m_cb_PostNew;

	// the callback ID's for open scene
	MCallbackId m_cb_PreOpen;
	MCallbackId m_cb_PostOpen;

	// the callback ID's for save scene
	MCallbackId m_cb_PreSave;
	MCallbackId m_cb_PostSave;



	// static export callback functions
	static void SPreExport(void* data);
	static void SPostExport(void* data);

	// static import callback functions
	static void SPreImport(void* data);
	static void SPostImport(void* data);

	// static new scene callback functions
	static void SPreNew(void* data);
	static void SPostNew(void* data);

	// static open scene callback functions
	static void SPreOpen(void* data);
	static void SPostOpen(void* data);

	// static save scene callback functions
	static void SPreSave(void* data);
	static void SPostSave(void* data);
};


//---------------------------------------------------------------------------------------------
//
void FileMenuWatcher::SPreExport(void* data) {
	( (FileMenuWatcher*)data )->PreExport();
}
void FileMenuWatcher::SPostExport(void* data) {
	( (FileMenuWatcher*)data )->PostExport();
}

//---------------------------------------------------------------------------------------------
//
void FileMenuWatcher::SPreImport(void* data) {
	( (FileMenuWatcher*)data )->PreImport();
}
void FileMenuWatcher::SPostImport(void* data) {
	( (FileMenuWatcher*)data )->PostImport();
}

//---------------------------------------------------------------------------------------------
//
void FileMenuWatcher::SPreNew(void* data) {
	( (FileMenuWatcher*)data )->PreNew();
}
void FileMenuWatcher::SPostNew(void* data) {
	( (FileMenuWatcher*)data )->PostNew();
}

//---------------------------------------------------------------------------------------------
//
void FileMenuWatcher::SPreOpen(void* data) {
	( (FileMenuWatcher*)data )->PreOpen();
}
void FileMenuWatcher::SPostOpen(void* data) {
	( (FileMenuWatcher*)data )->PostOpen();
}

//---------------------------------------------------------------------------------------------
//
void FileMenuWatcher::SPreSave(void* data) {
	( (FileMenuWatcher*)data )->PreSave();
}
void FileMenuWatcher::SPostSave(void* data) {
	( (FileMenuWatcher*)data )->PostSave();
}

//---------------------------------------------------------------------------------------------
//
FileMenuWatcher::FileMenuWatcher() {

	/// register all callbacks with Maya
	///
	m_cb_PreExport = MSceneMessage::addCallback( MSceneMessage::kBeforeExport, FileMenuWatcher::SPreExport, this );
	m_cb_PostExport = MSceneMessage::addCallback( MSceneMessage::kAfterExport, FileMenuWatcher::SPostExport, this );

	m_cb_PreImport = MSceneMessage::addCallback( MSceneMessage::kBeforeImport, FileMenuWatcher::SPreImport, this );
	m_cb_PostImport = MSceneMessage::addCallback( MSceneMessage::kAfterImport, FileMenuWatcher::SPostImport, this );

	m_cb_PreNew = MSceneMessage::addCallback( MSceneMessage::kBeforeNew, FileMenuWatcher::SPreNew, this );
	m_cb_PostNew = MSceneMessage::addCallback( MSceneMessage::kAfterNew, FileMenuWatcher::SPostNew, this );

	m_cb_PreOpen = MSceneMessage::addCallback( MSceneMessage::kBeforeOpen, FileMenuWatcher::SPreOpen, this );
	m_cb_PostOpen = MSceneMessage::addCallback( MSceneMessage::kAfterOpen, FileMenuWatcher::SPostOpen, this );

	m_cb_PreSave = MSceneMessage::addCallback( MSceneMessage::kBeforeSave, FileMenuWatcher::SPreSave, this );
	m_cb_PostSave = MSceneMessage::addCallback( MSceneMessage::kAfterSave, FileMenuWatcher::SPostSave, this );
}

//---------------------------------------------------------------------------------------------
//
FileMenuWatcher::~FileMenuWatcher() {
	/// make sure we delete the callbacks before the plugin quits !!!
	/// Maya is likely to crash otherwise!
	MSceneMessage::removeCallback( m_cb_PreExport );
	MSceneMessage::removeCallback( m_cb_PostExport );

	MSceneMessage::removeCallback( m_cb_PreImport );
	MSceneMessage::removeCallback( m_cb_PostImport );

	MSceneMessage::removeCallback( m_cb_PreNew );
	MSceneMessage::removeCallback( m_cb_PostNew );

	MSceneMessage::removeCallback( m_cb_PreOpen );
	MSceneMessage::removeCallback( m_cb_PostOpen );

	MSceneMessage::removeCallback( m_cb_PreSave );
	MSceneMessage::removeCallback( m_cb_PostSave );
}



/// pointer to the callback object
FileMenuWatcher* menuWatch = 0;



/// \brief	This function is called once when our plugin is loaded. We can use
///			it to register any custom nodes, mel functions etc.
/// \param	obj	-	a handle to the loaded plugin
/// \return	an MStatus error code
///
EXPORT MStatus initializePlugin( MObject obj )
{
	// we need to use the plugin function set to register a new mel command with maya.
	// we can provide some info about the author of the plugin, the version number, and we
	// can also specify a required version of maya, though usually "Any" will do...
	//
	MFnPlugin fnPlugin( obj, "Rob Bateman", "1.0", "Any");

	// create the file menu watcher
	menuWatch = new FileMenuWatcher;

	return MS::kSuccess;
}

/// \brief	This function is called once when our plugin is unloaded. We need
/// 		to tell maya which mel funcs, nodes etc we are removing.
/// \param	obj	-	a handle to the loaded plugin
/// \return	an MStatus error code
///
EXPORT MStatus uninitializePlugin( MObject obj)
{
	// use the plugin function set to unload the plugin
	MFnPlugin fnPlugin( obj );

	// delete the object
	delete menuWatch;

	return MS::kSuccess;
}


